@@ -1,17 +1,23 @@
-import asyncio
 import asyncio.events
 import errno
 import re
+import sys
 import threading
+import weakref
 import zmq
 
-from asyncio.unix_events import SelectorEventLoop, SafeChildWatcher
 from asyncio.transports import _FlowControlMixin
 from collections import deque, Iterable, Set
 from ipaddress import ip_address
 
-from .selector import ZmqSelector
 from .interface import ZmqTransport
+from .selector import ZmqSelector
+
+
+if sys.platform == 'win32':
+    from asyncio.windows_events import SelectorEventLoop
+else:
+    from asyncio.unix_events import SelectorEventLoop, SafeChildWatcher
 
 
 __all__ = ['ZmqEventLoop', 'ZmqEventLoopPolicy']
@@ -24,13 +30,19 @@ class ZmqEventLoop(SelectorEventLoop):
     create_zmq_connection method for working with ZeroMQ sockets.
     """
 
-    def __init__(self, *, io_threads=1):
+    def __init__(self, *, zmq_context=None):
         super().__init__(selector=ZmqSelector())
-        self._zmq_context = zmq.Context(io_threads)
+        if zmq_context is None:
+            self._zmq_context = zmq.Context.instance()
+        else:
+            self._zmq_context = zmq_context
+        self._zmq_sockets = weakref.WeakSet()
 
     def close(self):
+        for zmq_sock in self._zmq_sockets:
+            if not zmq_sock.closed:
+                zmq_sock.close()
         super().close()
-        self._zmq_context.destroy()
 
     @asyncio.coroutine
     def create_zmq_connection(self, protocol_factory, zmq_type, *,
@@ -104,7 +116,7 @@ class ZmqEventLoop(SelectorEventLoop):
                         raise ValueError('bind should be str or iterable')
                 for endpoint in bind:
                     yield from transport.bind(endpoint)
-            elif connect is not None:
+            if connect is not None:
                 if isinstance(connect, str):
                     connect = [connect]
                 else:
@@ -113,7 +125,7 @@ class ZmqEventLoop(SelectorEventLoop):
                                          'str or iterable')
                 for endpoint in connect:
                     yield from transport.connect(endpoint)
-
+            self._zmq_sockets.add(zmq_sock)
             return transport, protocol
         except OSError:
             # don't care if zmq_sock.close can raise exception
@@ -273,7 +285,8 @@ class _ZmqTransportImpl(ZmqTransport, _FlowControlMixin):
         try:
             self._protocol.connection_lost(exc)
         finally:
-            self._zmq_sock.close()
+            if not self._zmq_sock.closed:
+                self._zmq_sock.close()
             self._zmq_sock = None
             self._protocol = None
             self._loop = None
@@ -420,10 +433,9 @@ class ZmqEventLoopPolicy(asyncio.AbstractEventLoopPolicy):
         _loop = None
         _set_called = False
 
-    def __init__(self, *, io_threads=1):
+    def __init__(self):
         self._local = self._Local()
         self._watcher = None
-        self._io_threads = io_threads
 
     def get_event_loop(self):
         """Get the event loop.
@@ -451,15 +463,7 @@ class ZmqEventLoopPolicy(asyncio.AbstractEventLoopPolicy):
         You must call set_event_loop() to make this the current event
         loop.
         """
-        return ZmqEventLoop(io_threads=self._io_threads)
-
-    def _init_watcher(self):
-        with asyncio.events._lock:
-            if self._watcher is None:  # pragma: no branch
-                self._watcher = SafeChildWatcher()
-                if isinstance(threading.current_thread(),
-                              threading._MainThread):
-                    self._watcher.attach_loop(self._local._loop)
+        return ZmqEventLoop()
 
     def set_event_loop(self, loop):
         """Set the event loop.
@@ -478,24 +482,33 @@ class ZmqEventLoopPolicy(asyncio.AbstractEventLoopPolicy):
                 isinstance(threading.current_thread(), threading._MainThread)):
             self._watcher.attach_loop(loop)
 
-    def get_child_watcher(self):
-        """Get the child watcher.
+    if sys.platform != 'win32':
+        def _init_watcher(self):
+            with asyncio.events._lock:
+                if self._watcher is None:  # pragma: no branch
+                    self._watcher = SafeChildWatcher()
+                    if isinstance(threading.current_thread(),
+                                  threading._MainThread):
+                        self._watcher.attach_loop(self._local._loop)
 
-        If not yet set, a SafeChildWatcher object is automatically created.
-        """
-        if self._watcher is None:
-            self._init_watcher()
+        def get_child_watcher(self):
+            """Get the child watcher.
 
-        return self._watcher
+            If not yet set, a SafeChildWatcher object is automatically created.
+            """
+            if self._watcher is None:
+                self._init_watcher()
 
-    def set_child_watcher(self, watcher):
-        """Set the child watcher."""
+            return self._watcher
 
-        assert watcher is None or \
-            isinstance(watcher, asyncio.AbstractChildWatcher), \
-            "watcher should be None or AbstractChildWatcher instance"
+        def set_child_watcher(self, watcher):
+            """Set the child watcher."""
 
-        if self._watcher is not None:
-            self._watcher.close()
+            assert watcher is None or \
+                isinstance(watcher, asyncio.AbstractChildWatcher), \
+                "watcher should be None or AbstractChildWatcher instance"
 
-        self._watcher = watcher
+            if self._watcher is not None:
+                self._watcher.close()
+
+            self._watcher = watcher
